@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -32,7 +32,8 @@ namespace metaproxy_1 {
         public:
             ZeeRexExplain();
             ~ZeeRexExplain();
-            void configure(const xmlNode *xmlnode, bool test_only);
+            void configure(const xmlNode *xmlnode, bool test_only,
+                           const char *path);
             void process(metaproxy_1::Package & package) const;
         };
     }
