@@ -1,5 +1,5 @@
 /* This file is part of Metaproxy.
-   Copyright (C) 2005-2010 Index Data
+   Copyright (C) 2005-2012 Index Data
 
 Metaproxy is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -34,6 +34,8 @@ namespace metaproxy_1 {
         virtual ~RouterChain();
         virtual RoutePos *createpos() const;
         RouterChain & append(const filter::Base &filter);
+        void start();
+        void stop();
     private:
         boost::scoped_ptr<Rep> m_p;
         /// disabled because class is singleton
